import * as ai from 'ai';
import { LanguageModel, CoreMessage, CoreSystemMessage, CoreUserMessage, CoreAssistantMessage, CoreToolMessage, TextPart, ImagePart, FilePart, ToolCallPart, ToolCallRepairFunction, StepResult, CoreTool, CoreToolChoice, GenerateTextResult, StreamTextResult } from 'ai';
import { Resolvable } from '@synstack/resolved';
import { z } from 'zod';
import { Text } from '@synstack/text';

type $Partial<T> = Partial<T>;
declare namespace Llm {
    /**
     * Language model that is used by the AI SDK Core functions.
     */
    type Model = LanguageModel;
    /**
     * A message that can be used in the `messages` field of a prompt.
     * It can be a user message, an assistant message, or a tool message.
     */
    type Message = CoreMessage;
    namespace Message {
        /**
         * Serializable message parts.
         * Used for message templating with `userMsg` and `assistantMsg`.
         */
        namespace Template {
            type Part = any;
            namespace Part {
                type Image = Image.Base64 | Image.Path | Image.Url;
                namespace Image {
                    type Base64 = {
                        type: "image";
                        encoding: "base64";
                        mimeType: string;
                        data: string;
                    };
                    type Path = {
                        type: "image";
                        encoding: "path";
                        mimeType: string;
                        data: string;
                    };
                    type Url = {
                        type: "image";
                        encoding: "url";
                        mimeType: string;
                        data: string;
                    };
                }
                type File = File.Base64 | File.Path | File.Url;
                namespace File {
                    type Base64 = {
                        type: "file";
                        encoding: "base64";
                        mimeType: string;
                        data: string;
                    };
                    type Path = {
                        type: "file";
                        encoding: "path";
                        mimeType: string;
                        data: string;
                    };
                    type Url = {
                        type: "file";
                        encoding: "url";
                        mimeType: string;
                        data: string;
                    };
                }
            }
        }
        /**
         * A system message. It can contain system information.
         *
         * Note: using the "system" part of the prompt is strongly preferred
         * to increase the resilience against prompt injection attacks,
         * and because not all providers support several system messages.
         */
        type System = CoreSystemMessage;
        /**
         * A user message. It can contain text or a combination of text and images.
         */
        type User = CoreUserMessage;
        /**
         * An assistant message. It can contain text, tool calls, or a combination of text and tool calls.
         */
        type Assistant = CoreAssistantMessage;
        /**
         * A tool message. It contains the result of one or more tool calls.
         */
        type ToolResult = CoreToolMessage;
        /**
         * Content parts of a message in a prompt.
         */
        namespace Part {
            /**
             * Text content part of a prompt. It contains a string of text.
             */
            type Text = TextPart;
            /**
             * Image content part of a prompt. It contains an image.
             */
            type Image = ImagePart;
            /**
             * File content part of a prompt. It contains a file.
             */
            type File = FilePart;
            /**
             * Tool call content part of a prompt. It contains a tool call (usually generated by the AI model).
             */
            type ToolCall = ToolCallPart;
            /**
             * Content parts of a user message.
             */
            type User = TextPart | ImagePart | FilePart;
            /**
             * Content parts of an assistant message.
             */
            type Assistant = TextPart | ToolCall;
        }
    }
    type Completion = {
        /**
         * The language model to use.
         */
        model: Llm.Model;
        /**
         * The messages to use in the prompt.
         */
        messages: Resolvable<Array<Llm.Message>>;
        /**
         * Maximum number of tokens to generate.
         */
        maxTokens?: number;
        /**
         * Temperature setting.
         * The value is passed through to the provider. The range depends on the provider and model.
         * It is recommended to set either `temperature` or `topP`, but not both.
         */
        temperature?: number;
        /**
         * Nucleus sampling.
         * The value is passed through to the provider. The range depends on the provider and model.
         * It is recommended to set either `temperature` or `topP`, but not both.
         */
        topP?: number;
        /**
         * Only sample from the top K options for each subsequent token.
         * Used to remove "long tail" low probability responses.
         * Recommended for advanced use cases only. You usually only need to use temperature.
         */
        topK?: number;
        /**
         * Frequency penalty setting.
         * It affects the likelihood of the model to repeatedly use the same words or phrases.
         * The value is passed through to the provider. The range depends on the provider and model.
         */
        frequencyPenalty?: number;
        /**
         * Presence penalty setting.
         * It affects the likelihood of the model to repeat information that is already in the prompt.
         * The value is passed through to the provider. The range depends on the provider and model.
         */
        presencePenalty?: number;
        /**
         * The seed (integer) to use for random sampling.
         * If set and supported by the model, calls will generate deterministic results.
         */
        seed?: number;
        /**
         * The tools that the model can call. The model needs to support calling tools.
         */
        tools?: Llm.Tools;
        /**
         * The tool choice strategy. Default: 'auto'.
         */
        toolChoice?: Llm.ToolChoice<Record<string, Llm.Tool>>;
        /**
         * Stop sequences.
         * If set, the model will stop generating text when one of the stop sequences is generated.
         */
        stopSequences?: string[];
        /**
         * Maximum number of retries. Set to 0 to disable retries. Default: 2.
         */
        maxRetries?: number;
        /**
         * An optional abort signal that can be used to cancel the call.
         */
        abortSignal?: AbortSignal;
        /**
         * Maximum number of sequential LLM calls (steps), e.g. when you use tool calls. Must be at least 1.
         *
         * A maximum number is required to prevent infinite loops in the case of misconfigured tools.
         *
         * By default, it's set to 1, which means that only a single LLM call is made.
         */
        maxSteps?: number;
        /**
         * When enabled, the model will perform additional steps if the finish reason is "length" (experimental).
         *
         * By default, it's set to false.
         */
        experimental_continueSteps?: boolean;
        /**
         * Optional telemetry configuration (experimental).
         */
        experimental_telemetry?: any;
        /**
         * Additional provider-specific metadata. They are passed through
         * to the provider from the AI SDK and enable provider-specific
         * functionality that can be fully encapsulated in the provider.
         */
        experimental_providerMetadata?: any;
        /**
         * Limits the tools that are available for the model to call without
         * changing the tool call and result types in the result.
         */
        experimental_activeTools?: Array<string>;
        /**
         * A function that attempts to repair a tool call that failed to parse.
         */
        experimental_repairToolCalls?: ToolCallRepairFunction<Llm.Tools>;
        /**
         * Callback that is called when each step (LLM call) is finished, including intermediate steps.
         */
        onStepFinish?: (event: StepResult<Llm.Tools>) => Promise<void> | void;
    };
    namespace Completion {
        /**
         * Partial completion configuration.
         */
        type Partial = $Partial<Completion>;
    }
    /**
     * A tool contains the description and the schema of the input that the tool expects.
     * This enables the language model to generate the input.
     *
     * The tool can also contain an optional execute function for the actual execution function of the tool.
     */
    type Tool<PARAMETERS extends z.ZodTypeAny = any, RESULT = any> = CoreTool<PARAMETERS, RESULT>;
    /**
     * A record of Llm.Tool
     */
    type Tools<Key extends string = string, Tool extends Llm.Tool = Llm.Tool> = Record<Key, Tool>;
    /**
     * Tool choice for the generation. It supports the following settings:
     *
     * - `auto` (default): the model can choose whether and which tools to call.
     * - `required`: the model must call a tool. It can choose which tool to call.
     * - `none`: the model must not call tools
     * - `{ type: 'tool', toolName: string (typed) }`: the model must call the specified tool
     */
    type ToolChoice<Tools extends Record<string, CoreTool>> = CoreToolChoice<Tools>;
}

declare class CompletionBuilder<OPTIONS extends Llm.Completion.Partial> {
    /**
     * Create a new CompletionBuilder with an empty configuration.
     */
    static get new(): CompletionBuilder<{}>;
    /**
     * Create a new CompletionBuilder from an existing configuration.
     */
    static from<T extends Llm.Completion.Partial>(options?: T): CompletionBuilder<T extends undefined ? {} : T>;
    private readonly _options;
    private constructor();
    /**
     * Merge current options with another configuration
     */
    merge<NEW_OPTIONS extends Llm.Completion.Partial>(options: NEW_OPTIONS): CompletionBuilder<{ [K in keyof OPTIONS | keyof NEW_OPTIONS]: K extends keyof NEW_OPTIONS ? NEW_OPTIONS[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Get the current options as a plain object.
     */
    get options(): OPTIONS;
    /**
     * Get the current options as a plain object.
     */
    valueOf(): OPTIONS;
    /**
     * Set the language model to use.
     */
    model<VALUE extends Llm.Model>(model: VALUE): CompletionBuilder<{ [K in "model" | keyof OPTIONS]: K extends "model" ? {
        model: VALUE;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Set the messages to use in the prompt.
     */
    messages(messages: Resolvable.ArrayOf<Llm.Message>): CompletionBuilder<{ [K in "messages" | keyof OPTIONS]: K extends "messages" ? {
        messages: Promise<ai.CoreMessage[]>;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Append messages to the existing prompt messages.
     */
    appendMessages(messages: Array<Llm.Message>): CompletionBuilder<{ [K in "messages" | keyof OPTIONS]: K extends "messages" ? {
        messages: Promise<ai.CoreMessage[]>;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Set the tools that the model can call. The model needs to support calling tools.
     */
    tools<NEW_TOOLS extends Llm.Tools>(tools: NEW_TOOLS): CompletionBuilder<{ [K in "tools" | keyof OPTIONS]: K extends "tools" ? {
        tools: NEW_TOOLS;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Set the tool choice strategy. Default: 'auto'.
     *
     * Supports the following settings:
     * - `auto` (default): the model can choose whether and which tools to call.
     * - `required`: the model must call a tool. It can choose which tool to call.
     * - `none`: the model must not call tools
     * - `{ type: 'tool', toolName: string (typed) }`: the model must call the specified tool
     */
    toolChoice<VALID_OPTIONS extends Llm.Completion.Partial & {
        tools: Llm.Tools;
    }, TOOL_CHOICE extends Llm.ToolChoice<VALID_OPTIONS["tools"]>>(this: CompletionBuilder<VALID_OPTIONS>, toolChoice: TOOL_CHOICE): CompletionBuilder<{ [K in "toolChoice" | keyof VALID_OPTIONS]: K extends "toolChoice" ? {
        toolChoice: Llm.ToolChoice<Llm.Tools>;
    }[K] : K extends keyof VALID_OPTIONS ? VALID_OPTIONS[K] : never; }>;
    /**
     * Set a function that attempts to repair a tool call that failed to parse.
     * @experimental
     */
    repairToolCalls(repairToolCalls: ToolCallRepairFunction<Llm.Tools>): CompletionBuilder<{ [K in "experimental_repairToolCalls" | keyof OPTIONS]: K extends "experimental_repairToolCalls" ? {
        experimental_repairToolCalls: ToolCallRepairFunction<Llm.Tools<string, Llm.Tool<any, any>>>;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Limit the tools that are available for the model to call without
     * changing the tool call and result types in the result.
     * @experimental
     */
    activeTools<VALID_OPTIONS extends Llm.Completion.Partial & {
        tools: Llm.Tools;
    }, ACTIVE_TOOLS extends Array<keyof VALID_OPTIONS["tools"]>>(this: CompletionBuilder<VALID_OPTIONS>, activeTools: ACTIVE_TOOLS): CompletionBuilder<{ [K in "experimental_activeTools" | keyof VALID_OPTIONS]: K extends "experimental_activeTools" ? {
        experimental_activeTools: string[];
    }[K] : K extends keyof VALID_OPTIONS ? VALID_OPTIONS[K] : never; }>;
    /**
     * Maximum number of tokens to generate.
     */
    maxTokens(maxTokens: number): CompletionBuilder<{ [K in "maxTokens" | keyof OPTIONS]: K extends "maxTokens" ? {
        maxTokens: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Temperature setting.
     * The value is passed through to the provider. The range depends on the provider and model.
     * It is recommended to set either `temperature` or `topP`, but not both.
     */
    temperature(temperature: number): CompletionBuilder<{ [K in "temperature" | keyof OPTIONS]: K extends "temperature" ? {
        temperature: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Nucleus sampling.
     * The value is passed through to the provider. The range depends on the provider and model.
     * It is recommended to set either `temperature` or `topP`, but not both.
     */
    topP(topP: number): CompletionBuilder<{ [K in "topP" | keyof OPTIONS]: K extends "topP" ? {
        topP: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Only sample from the top K options for each subsequent token.
     * Used to remove "long tail" low probability responses.
     * Recommended for advanced use cases only. You usually only need to use temperature.
     */
    topK(topK: number): CompletionBuilder<{ [K in "topK" | keyof OPTIONS]: K extends "topK" ? {
        topK: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Frequency penalty setting.
     * It affects the likelihood of the model to repeatedly use the same words or phrases.
     * The value is passed through to the provider. The range depends on the provider and model.
     */
    frequencyPenalty(frequencyPenalty: number): CompletionBuilder<{ [K in "frequencyPenalty" | keyof OPTIONS]: K extends "frequencyPenalty" ? {
        frequencyPenalty: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Presence penalty setting.
     * It affects the likelihood of the model to repeat information that is already in the prompt.
     * The value is passed through to the provider. The range depends on the provider and model.
     */
    presencePenalty(presencePenalty: number): CompletionBuilder<{ [K in "presencePenalty" | keyof OPTIONS]: K extends "presencePenalty" ? {
        presencePenalty: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * The seed (integer) to use for random sampling.
     * If set and supported by the model, calls will generate deterministic results.
     */
    seed(seed: number): CompletionBuilder<{ [K in "seed" | keyof OPTIONS]: K extends "seed" ? {
        seed: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Stop sequences.
     * If set, the model will stop generating text when one of the stop sequences is generated.
     */
    stopSequences(stopSequences: string[]): CompletionBuilder<{ [K in "stopSequences" | keyof OPTIONS]: K extends "stopSequences" ? {
        stopSequences: string[];
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Maximum number of retries. Set to 0 to disable retries. Default: 2.
     */
    maxRetries(maxRetries: number): CompletionBuilder<{ [K in "maxRetries" | keyof OPTIONS]: K extends "maxRetries" ? {
        maxRetries: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * An optional abort signal that can be used to cancel the call.
     */
    abortSignal(abortSignal: AbortSignal): CompletionBuilder<{ [K in "abortSignal" | keyof OPTIONS]: K extends "abortSignal" ? {
        abortSignal: AbortSignal;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Maximum number of sequential LLM calls (steps), e.g. when you use tool calls. Must be at least 1.
     *
     * A maximum number is required to prevent infinite loops in the case of misconfigured tools.
     *
     * By default, it's set to 1, which means that only a single LLM call is made.
     */
    maxSteps(maxSteps: number): CompletionBuilder<{ [K in "maxSteps" | keyof OPTIONS]: K extends "maxSteps" ? {
        maxSteps: number;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * When enabled, the model will perform additional steps if the finish reason is "length" (experimental).
     *
     * By default, it's set to false.
     * @experimental
     */
    continueSteps(continueSteps: boolean): CompletionBuilder<{ [K in "experimental_continueSteps" | keyof OPTIONS]: K extends "experimental_continueSteps" ? {
        experimental_continueSteps: boolean;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Optional telemetry configuration.
     * @experimental
     */
    telemetry(telemetry: any): CompletionBuilder<{ [K in "experimental_telemetry" | keyof OPTIONS]: K extends "experimental_telemetry" ? {
        experimental_telemetry: any;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Additional provider-specific metadata. They are passed through
     * to the provider from the AI SDK and enable provider-specific
     * functionality that can be fully encapsulated in the provider.
     * @experimental
     */
    providerMetadata(providerMetadata: any): CompletionBuilder<{ [K in "experimental_providerMetadata" | keyof OPTIONS]: K extends "experimental_providerMetadata" ? {
        experimental_providerMetadata: any;
    }[K] : K extends keyof OPTIONS ? OPTIONS[K] : never; }>;
    /**
     * Generate a text and call tools for a given prompt using a language model.
     *
     * This function does not stream the output. If you want to stream the output, use `streamText` instead.
     *
     * @returns A result object that contains the generated text, the results of the tool calls, and additional information.
     */
    generateText<VALID_OPTIONS extends Llm.Completion & {
        tools?: Llm.Tools;
    }, TOOLS extends VALID_OPTIONS["tools"]>(this: CompletionBuilder<VALID_OPTIONS>): Promise<TOOLS extends Llm.Tools ? GenerateTextResult<TOOLS, never> : GenerateTextResult<never, never>>;
    /**
     * Generate a text and call tools for a given prompt using a language model.
     *
     * This function streams the output. If you do not want to stream the output, use `generateText` instead.
     *
     * @returns A result object for accessing different stream types and additional information.
     */
    streamText<VALID_OPTIONS extends Llm.Completion & {
        tools?: Llm.Tools;
    }, TOOLS extends VALID_OPTIONS["tools"]>(this: CompletionBuilder<VALID_OPTIONS>): TOOLS extends Llm.Tools ? Promise<StreamTextResult<TOOLS>> : Promise<StreamTextResult<never>>;
    /**
     * Generate a structured, typed object for a given prompt and schema using a language model.
     *
     * This function does not stream the output. If you want to stream the output, use `streamObject` instead.
     *
     * @returns A result object that contains the generated object, the finish reason, the token usage, and additional information.
     */
    generateObject<OBJECT, VALID_OPTIONS extends Llm.Completion>(this: CompletionBuilder<VALID_OPTIONS>, options: ObjectOptions<OBJECT>): Promise<ai.GenerateObjectResult<OBJECT>>;
    /**
     * Generate a structured, typed object for a given prompt and schema using a language model.
     *
     * This function streams the output. If you do not want to stream the output, use `generateObject` instead.
     *
     * @returns A result object for accessing the partial object stream and additional information.
     */
    streamObject<OBJECT, VALID_OPTIONS extends Llm.Completion>(this: CompletionBuilder<VALID_OPTIONS>, options: ObjectOptions<OBJECT>): Promise<ai.StreamObjectResult<ai.DeepPartial<OBJECT>, OBJECT, never>>;
}
interface ObjectOptions<T> {
    schema: z.Schema<T>;
    name?: string;
    description?: string;
}
declare const completion: CompletionBuilder<{}>;

declare const userMsg: <T extends Array<MessageTemplate.User.TemplateValue> = MessageTemplate.User.TemplateValue[]>(template: TemplateStringsArray, ...values: T) => {
    role: "user";
    content: ({
        type: "text";
        text: string;
        mimeType?: undefined;
        image?: undefined;
        data?: undefined;
    } | {
        type: "image";
        mimeType: string;
        image: string | Buffer<ArrayBufferLike>;
        text?: undefined;
        data?: undefined;
    } | {
        type: "file";
        mimeType: string;
        data: string | Buffer<ArrayBufferLike>;
        text?: undefined;
        image?: undefined;
    })[];
} | Promise<{
    role: "user";
    content: ({
        type: "text";
        text: string;
        mimeType?: undefined;
        image?: undefined;
        data?: undefined;
    } | {
        type: "image";
        mimeType: string;
        image: string | Buffer<ArrayBufferLike>;
        text?: undefined;
        data?: undefined;
    } | {
        type: "file";
        mimeType: string;
        data: string | Buffer<ArrayBufferLike>;
        text?: undefined;
        image?: undefined;
    })[];
}>;
declare const assistantMsg: <T extends Array<MessageTemplate.Assistant.TemplateValue> = MessageTemplate.Assistant.TemplateValue[]>(template: TemplateStringsArray, ...values: T) => {
    role: "assistant";
    content: (Text.ExtraObject.Infer<T> | {
        type: "text";
        text: string;
    })[] | (Awaited<Text.ExtraObject.Infer<T>> | {
        type: "text";
        text: string;
    })[];
} | Promise<{
    role: "assistant";
    content: (Text.ExtraObject.Infer<T> | {
        type: "text";
        text: string;
    })[] | (Awaited<Text.ExtraObject.Infer<T>> | {
        type: "text";
        text: string;
    })[];
}>;
declare const systemMsg: <T extends Array<MessageTemplate.System.TemplateValue> = MessageTemplate.System.TemplateValue[]>(template: TemplateStringsArray, ...values: T) => {
    role: "system";
    content: (string & {
        __extra: Text.ExtraObject.Infer<T>;
    }) | (string & {
        __extra: Text.ExtraObject.Infer<T>;
    });
} | Promise<{
    role: "system";
    content: (string & {
        __extra: Text.ExtraObject.Infer<T>;
    }) | (string & {
        __extra: Text.ExtraObject.Infer<T>;
    });
}>;
declare namespace MessageTemplate {
    type Fn<TExtraValue extends Text.ExtraObject.Base = never> = (template: TemplateStringsArray, ...values: Array<Text.TemplateValue<TExtraValue>>) => void;
    namespace System {
        type ExtraValues = never;
        type TemplateValue = Text.TemplateValue<ExtraValues>;
        type Fn = MessageTemplate.Fn<ExtraValues>;
    }
    namespace Assistant {
        type ExtraValues = Llm.Message.Part.ToolCall;
        type TemplateValue = Text.TemplateValue<ExtraValues>;
        type Fn = MessageTemplate.Fn<ExtraValues>;
    }
    namespace User {
        type ExtraValues = Llm.Message.Template.Part.Image | Llm.Message.Template.Part.File;
        type TemplateValue = Text.TemplateValue<ExtraValues>;
        type Fn = MessageTemplate.Fn<ExtraValues>;
    }
}

declare const filePart: {
    fromPath(path: string, mimeType?: string): Llm.Message.Template.Part.Image | Llm.Message.Template.Part.File;
    fromBase64(base64: string, mimeType?: string): Llm.Message.Template.Part.Image | Llm.Message.Template.Part.File;
    fromUrl(url: string, mimeType?: string): {
        type: string;
        encoding: string;
        mimeType: string | undefined;
        url: string;
    };
};

export { CompletionBuilder, Llm, MessageTemplate, assistantMsg, completion, filePart, systemMsg, userMsg };
